/**
 * Universitat Oberta de Catalunya
 * Ingeniera de Informtica
 * Proyecto fin de carrera
 * 
 * @author      Miguel Arias Fernndez 
 * @version     1.0    
 * 
 */
package es.uoc.maf.core.action.utils;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.LocaleUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.uoc.maf.core.Constants;
import es.uoc.maf.core.action.Action;
import es.uoc.maf.core.config.ActionConfig;
import es.uoc.maf.core.config.Configuration;
import es.uoc.maf.core.config.ErrorHandlerConfig;
import es.uoc.maf.core.config.FormBeanConfig;
import es.uoc.maf.core.config.ResultTypeConfig;
import es.uoc.maf.core.error.ErrorHandler;
import es.uoc.maf.core.exceptions.ActionCreationException;
import es.uoc.maf.core.exceptions.ActionNotFound;
import es.uoc.maf.core.exceptions.ErrorHandlerCreationException;
import es.uoc.maf.core.exceptions.FormBeanCreationException;
import es.uoc.maf.core.exceptions.FormNotFoundException;
import es.uoc.maf.core.exceptions.NoResultException;
import es.uoc.maf.core.exceptions.ResultCreationException;
import es.uoc.maf.core.form.FormBean;
import es.uoc.maf.core.form.utils.BeanUtils;
import es.uoc.maf.core.result.ResultType;

/**
 * The Class ActionProcessor.
 * 
 * @author Miguel Arias Fernndez
 * @version 1.0
 */
public abstract class ActionProcessor {
	
	/** The log. */
	private static Logger LOG = LoggerFactory.getLogger(ActionProcessor.class);

	/**
	 * Process action.
	 * 
	 * @param actionContext
	 *            the action context
	 * @param configuration
	 *            the configuration
	 */
	public static void processAction(ActionContext actionContext, Configuration configuration) {
		Action action = null;
		ActionConfig actionConfig = null;
		try {
			LOG.debug("iniciando accin.");
			// Creamos las variables ThreadLocal
			actionConfig = getActionConfig(actionContext.getRequest(), configuration);
			// Procesamos el formulario
			FormBean form = processForm(actionConfig, actionContext, configuration);
			// Instanciamos la accin
			action = createActionInstance(actionConfig);
			// La guardamos en la request, para ms adelante, acceder por ejemplo desde los TAGS
			actionContext.getRequest().setAttribute(Constants.ACTION_KEY, action);
			// Almacenamos en la accion el contexto, por si se necesita acceder a la request, por ejemplo...
			action.setActionContext(actionContext);
			// Establecemos el formulario
			action.setForm(form);
			// Ejecutamos la accion
			String result = action.execute();
			// Procesamos el resultado
			generateResult(actionConfig, actionContext, configuration, result);
		} catch (Throwable t) {
			LOG.debug("Error detectado procesando accin: {}",t.getMessage(),t);
			processError(t,action,actionContext, actionConfig, configuration);
		} finally {
			LOG.debug("Fin accin.");
		}
	}

	/**
	 * Process error.
	 * 
	 * @param error
	 *            the error
	 * @param action
	 *            the action
	 * @param actionContext
	 *            the action context
	 * @param actionConfig
	 *            the action config
	 * @param configuration
	 *            the configuration
	 */
	private static void processError(Throwable error, Action action, ActionContext actionContext, ActionConfig actionConfig, Configuration configuration) {
		LOG.debug("Procesando error: {} [{}]",error.getMessage(),error.getClass());
		ErrorHandlerConfig errorHandlerConfig = retrieveErrorHandler(error,
				actionConfig, configuration);
		LOG.debug("Error handler encontrado: {}",errorHandlerConfig);
		Class<ErrorHandler> errorHandlerClass = errorHandlerConfig.getErrorHandlerClass();
		ErrorHandler errorHandler;
		try {
			errorHandler = errorHandlerClass.newInstance();
		} catch (Throwable t) {
			LOG.debug("Error instanciando handler de tipo '{}'", errorHandlerClass.getName(), t);
			throw new ErrorHandlerCreationException(errorHandlerClass, t);
		}
		try {
			BeanUtils.getInstance().populate( errorHandler, errorHandlerConfig.getParameters(),LocaleUtils.toLocale(Constants.DEFAULT_LOCALE));
		} catch (Exception e) {
			LOG.debug("Error estableciendo parmetros de resultado de tipo '{}'", errorHandlerClass.getName(), e);
			throw new ErrorHandlerCreationException(errorHandlerClass, e);
		} 
		try {
			errorHandler.handle(error, action, actionContext);
		} catch (Exception e) {
			LOG.debug("Error generando resultado de tipo '{}'", errorHandlerClass.getName(), e);
			throw new ErrorHandlerCreationException(errorHandlerClass, e);
		}
	}

	/**
	 * Retrieve error handler.
	 * 
	 * @param error
	 *            the error
	 * @param actionConfig
	 *            the action config
	 * @param configuration
	 *            the configuration
	 * @return the error handler config
	 */
	@SuppressWarnings("unchecked")
	private static ErrorHandlerConfig retrieveErrorHandler(Throwable error,
			ActionConfig actionConfig, Configuration configuration) {
		Class<Throwable> errorClass = (Class<Throwable>) error.getClass();
		Map<Class<Throwable>, ErrorHandlerConfig> globalErrors = configuration.getGlobalErrors();
		while (errorClass != null) {
			ErrorHandlerConfig errorHandlerConfig = null;
			if (actionConfig != null) {
				// Por si an no est creado la configuracin de la accin...
				Map<Class<Throwable>, ErrorHandlerConfig> errorsMap = actionConfig.getErrorsConfig();
				errorHandlerConfig = errorsMap == null ? null : errorsMap.get(errorClass);
				if (errorHandlerConfig != null) {
					return errorHandlerConfig;
				}
				LOG.debug("No es un error local de la accin. Comprobando errores globales...");
			}
			else {
				LOG.debug("An no se ha creado la configuracin de la accin.");
			}
			errorHandlerConfig = globalErrors == null ? null : globalErrors.get(errorClass);
			if (errorHandlerConfig != null) {
				return errorHandlerConfig;
			}
			errorClass = (Class<Throwable>) errorClass.getSuperclass();
			LOG.debug("Error handler no encontrado. Comprobando superclase de error: {}",errorClass);
		}
		LOG.debug("Error handler NO encontrado para {}",error.getClass());
		throw new RuntimeException(error);
	}

	/**
	 * Generate result.
	 * 
	 * @param actionConfig
	 *            the action config
	 * @param actionContext
	 *            the action context
	 * @param configuration
	 *            the configuration
	 * @param result
	 *            the result
	 */
	private static void generateResult(ActionConfig actionConfig, ActionContext actionContext, Configuration configuration, String result) {
		LOG.debug("Procesando resultado de accin: {}", result);
		Map<String, ResultTypeConfig> resultsMap = actionConfig
				.getResultTypesConfig();
		// Obtenemos el resultado de la accin...
		ResultTypeConfig resultTypeConfig = resultsMap == null ? null : resultsMap.get(result);
		if (resultTypeConfig == null) {
			LOG.debug("No es un resultado propio de la accin. Comprobando resultados globales...");
			// ... si no est en los resultados propios, comprobamos los
			// globales...
			Map<String, ResultTypeConfig> globalResults = configuration.getGlobalResults();
			resultTypeConfig = globalResults == null ? null : globalResults.get(result);
		}
		LOG.debug("Resultado encontrado: {}", resultTypeConfig);
		if (resultTypeConfig == null) {
			throw new NoResultException(result, actionConfig.getName());
		}
		Class<ResultType> resultTypeClass = resultTypeConfig.getResultTypeClass();
		ResultType resultType;
		try {
			resultType = resultTypeClass.newInstance();
		} catch (Throwable t) {
			LOG.debug("Error instanciando resultado de tipo '{}'", resultTypeClass.getName(), t);
			throw new ResultCreationException(resultTypeClass, t);
		}
		try {
			BeanUtils.getInstance().populate( resultType, resultTypeConfig.getParameters(),LocaleUtils.toLocale(Constants.DEFAULT_LOCALE));
		} catch (Exception e) {
			LOG.debug("Error estableciendo parmetros de resultado de tipo '{}'", resultTypeClass.getName(), e);
			throw new ResultCreationException(resultTypeClass, e);
		} 
		try {
			resultType.generate(actionContext);
		} catch (Exception e) {
			LOG.debug("Error generando resultado de tipo '{}'", resultTypeClass.getName(), e);
			throw new ResultCreationException(resultTypeClass, e);
		} 
	}

	/**
	 * Creates the action instance.
	 * 
	 * @param actionConfig
	 *            the action config
	 * @return the action
	 */
	private static Action createActionInstance(ActionConfig actionConfig) {
		Class<Action> actionClass = actionConfig.getActionClass();
		LOG.debug("Action class: {}", actionClass);
		Action action;
		try {
			action = actionClass.newInstance();
		} catch (Exception e) {
			LOG.debug("Error creando accin '{}'", actionClass, e);
			throw new ActionCreationException(actionClass, e);
		}
		return action;
	}

	/**
	 * Gets the action config.
	 * 
	 * @param request
	 *            the request
	 * @param configuration
	 *            the configuration
	 * @return the action config
	 */
	private static ActionConfig getActionConfig(HttpServletRequest request, Configuration configuration) {
		// Recuperamos la configuracin de la accin
		String actionPath = StringUtils.defaultIfBlank(request.getPathInfo(), request.getServletPath());
		LOG.debug("Action path: " + actionPath);
		ActionConfig actionConfig = configuration.getActionMappings().get(actionPath);
		LOG.debug("Action config asociada: " + actionConfig);
		if (actionConfig == null) {
			throw new ActionNotFound(actionPath);
		}
		return actionConfig;
	}

	/**
	 * Process form.
	 * 
	 * @param actionConfig
	 *            the action config
	 * @param actionContext
	 *            the action context
	 * @param configuration
	 *            the configuration
	 * @return the form bean
	 */
	@SuppressWarnings("unchecked")
	private static FormBean processForm(ActionConfig actionConfig, ActionContext actionContext, Configuration configuration) {
		// Si hay un formulario asociado a la accion
		String formName = actionConfig.getFormName();
		if (formName == null) {
			LOG.debug("No hay formulario asociado a la accin.");
			return null;			
		}
		LOG.debug("Nombre del formulario asociado a la accin: [{}]", formName);
		FormBeanConfig actionFormConfig = configuration.getFormBeans().get(formName);
		LOG.debug("ActionFormConfig: {}", ReflectionToStringBuilder.toString(
				actionFormConfig, ToStringStyle.SHORT_PREFIX_STYLE));
		if (actionFormConfig == null) {
			LOG.debug("Formulario no encontrado: [{}]", formName);
			throw new FormNotFoundException(formName);
		}
		Class<FormBean> actionFormClass = actionFormConfig.getFormClass();
		LOG.debug("Clase del formulario [{}]: {}", formName, actionFormClass);
		FormBean form;
		try {
			form = actionFormClass.newInstance();
		} catch (Throwable t) {
			LOG.debug("Error creando formulario '{}'", formName, t);
			throw new FormBeanCreationException(actionFormClass, t);
		}
		try {
			BeanUtils.getInstance().populate(form, actionContext.getRequest().getParameterMap(),ActionContext.getLocale());
			form.validate();
			LOG.debug("FormErrors? {}: {}", form.hasErrors(), form.getFormErrors());

		} catch (Exception e) {
			LOG.debug(
					"Error poblando formulario '" + formName + "': "+ e.getLocalizedMessage(), e);
			LOG.error("Error poblando formulario '{}': {}", formName,
					e.getLocalizedMessage());
		}
		return form;
	}

}
