/**
 * Universitat Oberta de Catalunya
 * Ingeniera de Informtica
 * Proyecto fin de carrera
 * 
 * @author      Miguel Arias Fernndez 
 * @version     1.0    
 * 
 */
package es.uoc.maf.core.config;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import es.uoc.maf.core.Constants;
import es.uoc.maf.core.exceptions.CoreException;
import es.uoc.maf.core.servlet.CoreServlet;

/**
 * Factoria para crear Configuration.
 *
 * @author      Miguel Arias Fernndez 
 * @version     1.0    
 *
 */
public abstract class ConfigurationFactory {
	
	/** The Constant log. */
	private static final Logger log = LoggerFactory.getLogger(ConfigurationFactory.class);
	
	/** The _instance. */
	private static ConfigurationFactory _instance;
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException(ConfigurationFactory.class.getCanonicalName());
	}

	/**
	 * Gets the single instance of ConfigurationFactory.
	 * 
	 * @return single instance of ConfigurationFactory
	 */
	public static ConfigurationFactory getInstance() {
		if (_instance == null) {
			synchronized (ConfigurationFactory.class) {
				if (_instance == null) {
					String configFactoryClassName = CoreServlet.getContext().getInitParameter(Constants.CONFIG_FACTORY_CLASSNAME_PARAMETER);
					log.debug("Iniciando factora de configuracin. Clase especificada en el parmetro '{}' del servlet: {}",Constants.CONFIG_FACTORY_CLASSNAME_PARAMETER,configFactoryClassName);
					String className = StringUtils.defaultIfBlank(
							configFactoryClassName, 
							Constants.DEFAULT_FACTORY_CLASSNAME);
					log.debug("Nombre de clase a usar para factora de configuracin: {}",className);
					try {
						@SuppressWarnings("unchecked")
						Class<? extends ConfigurationFactory> clazz = (Class<? extends ConfigurationFactory>) Class.forName(className);
						_instance = clazz.newInstance();
						log.debug("Clase de factora de configuracin: {}",clazz.getName());
					}
					catch (ClassCastException cce) {
						log.error("La clase {} no implementa {}",className,ConfigurationFactory.class.getName());
					}
					catch (Exception e) {
						log.error("Clase de factoria de configuracin no encontrada: {}",e.getMessage());
						throw new CoreException("Error creando factora de configuracin ("+e.getClass()+"): "+e.getMessage(),e);
					}
				}
			}
		}
		return _instance;
	}
	
	/**
	 * Inits the config.
	 * 
	 * @return the configuration
	 */
	public Configuration initConfig() {
		log.debug("Iniciando configuracin de framework...");
		String configFileParameter = CoreServlet.getContext().getInitParameter(Constants.CONFIG_FILE_PARAMETER);

		log.debug("Fichero especificado en el parmetro '{}' del servlet: {}", Constants.CONFIG_FILE_PARAMETER, configFileParameter);
		String strConfigFile = StringUtils.defaultIfBlank(configFileParameter, Constants.DEFAULT_CONFIG_FILE);
		
		log.debug("Cargando configuracin de archivo [{}]",strConfigFile);
		ClassLoader cl = getClassLoader();
		log.trace("Utilizando ClassLoader {}"+cl.getClass());
		
		URL url = cl.getResource(strConfigFile);
		log.debug("Utilizando archivo [{}]",url);
		InputStream is = cl.getResourceAsStream(strConfigFile);
		if (is == null) {
			log.error("No se encuentra el archivo de configuracin {}",strConfigFile);
			throw new CoreException("No se encuentra el archivo de configuracin: "+strConfigFile);
		}
		try {
			validateConfig(is);
			// El stream se cierra en el validator.validate(), por lo que hay que volver a recuperarlo.
			return loadConfig(is = cl.getResourceAsStream(strConfigFile));
		}
		catch (Throwable t) {
			String mensaje = "Error cargando configuracin: "+t.getMessage();
			log.error(mensaje);
			log.debug(mensaje,t);
			throw new CoreException(mensaje, t);
		}
		finally {
			try {
				is.close();
			}
			catch (Exception e) {
				// Se ignora.
				log.debug("Error cerrando flujo: "+e.getMessage(),e);
			}
		}
	}

	/**
	 * Gets the class loader.
	 * 
	 * @return the class loader
	 */
	private ClassLoader getClassLoader() {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl == null) {
			cl = ConfigurationFactory.class.getClassLoader();
		}
		return cl;
	}

	/**
	 * Validate config.
	 * 
	 * @param is
	 *            the is
	 * @throws SAXException
	 *             the sAX exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void validateConfig(InputStream is) throws SAXException, IOException {
        SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        URL configSchemaFile = getClassLoader().getResource(Constants.CONFIG_SCHEMA_FILE);
        log.debug("Validando archivo de configuracin contra schema: {}",configSchemaFile);
        Schema schema = factory.newSchema(configSchemaFile);
		Validator validator = schema.newValidator();
        validator.validate(new StreamSource(is));
	}

	/**
	 * Load config.
	 * 
	 * @param is
	 *            the is
	 * @return the configuration
	 * @throws Exception
	 *             the exception
	 */
	protected abstract Configuration loadConfig(InputStream is) throws Exception;

}
