/**
 * Universitat Oberta de Catalunya
 * Ingeniera de Informtica
 * Proyecto fin de carrera
 * 
 * @author      Miguel Arias Fernndez 
 * @version     1.0    
 * 
 */
package es.uoc.maf.core.locale;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.uoc.maf.core.action.utils.ActionContext;

/**
 * The Class I18N.
 * 
 * @author Miguel Arias Fernndez
 * @version 1.0
 */
public class I18N {
	
	/** The log. */
	private static Logger LOG = LoggerFactory.getLogger(I18N.class);
	
	/** The Constant PATRON_PARAMS. */
	private static final Pattern PATRON_PARAMS = Pattern.compile("\\{\\d\\}");

	/** The Constant resourcesMap. */
	private static final ConcurrentHashMap<String, ConcurrentHashMap<Locale, ResourceBundle>> resourcesMap = new ConcurrentHashMap<String, ConcurrentHashMap<Locale,ResourceBundle>>();

	/**
	 * Instantiates a new i18 n.
	 */
	private I18N(){
	}
	
	/**
	 * Gets the string.
	 * 
	 * @param bundleName
	 *            the bundle name
	 * @param key
	 *            the key
	 * @param parametros
	 *            the parametros
	 * @return the string
	 */
	public static String getString(String bundleName, String key, String...parametros) {
		return getString(bundleName, key, ActionContext.getLocale(),parametros);
	}
	
	/**
	 * Gets the string.
	 * 
	 * @param bundleName
	 *            the bundle name
	 * @param key
	 *            the key
	 * @param locale
	 *            the locale
	 * @param parametros
	 *            the parametros
	 * @return the string
	 */
	public static String getString(String bundleName, String key, Locale locale, String... parametros) {
		LOG.debug("Buscando clave '{}' en recurso '{}' con idioma '{}'", key, bundleName, locale);
		// Primero buscamos si ya estn cargados los recursos ...
		ConcurrentHashMap<Locale, ResourceBundle> bundleMap = resourcesMap.get(bundleName);
		if (bundleMap == null) {
			// ... si no estn cargados, se cargan
			bundleMap = new ConcurrentHashMap<Locale, ResourceBundle>();
			resourcesMap.put(bundleName, bundleMap);
		}
		// Una vez tenemos el bundle, vemos si esta cargado el locale... 
		ResourceBundle resourceBundle = bundleMap.get(locale);
		if (resourceBundle == null) {
			// ... si no es as, lo cargamos
			resourceBundle = ResourceBundle.getBundle(bundleName, locale);
			bundleMap.put(locale, resourceBundle);
		}
		// Retornamos el valor de la cadena
		try {
			// Si no existe el recurso, se devuelve la clave directamente
			if (resourceBundle == null) {
				LOG.warn("Fichero de recursos '{}' no encontrado.",bundleName);
				return key;
			}
			// Sustituimos los parmetros
			StringBuffer sb = new StringBuffer();
			Matcher m = PATRON_PARAMS.matcher(resourceBundle.getString(key));
			// Sustituimos la coincidencia {<numero>} por el parametro correspondiente
			while (m.find()) {
				String coindicendia = m.group();
				int indice = Integer.valueOf(StringUtils.mid(coindicendia, 1, coindicendia.length() - 2));
				String valorParam = parametros == null || parametros.length > indice ? "" : parametros[indice];
				m.appendReplacement(sb, valorParam);
			}
			// aadimos desde la ultima coincidencia hasta el final (todo si no hay coincidencia)
			m.appendTail(sb);
			String value = sb.toString();
			LOG.debug("Valor encontrado: '{}'",value);
			return value;
		}
		catch (MissingResourceException mre) {
			LOG.debug("Clave '{}' no encontrada.",key);
			return key;
		}
	}

	/**
	 * Gets the string.
	 * 
	 * @param clazz
	 *            the clazz
	 * @param key
	 *            the key
	 * @param parametros
	 *            the parametros
	 * @return the string
	 */
	public static String getString(Class<?> clazz, String key, String... parametros) {
		return getString(clazz, key, ActionContext.getLocale(), parametros);
	}
	
	/**
	 * Gets the string.
	 * 
	 * @param clazz
	 *            the clazz
	 * @param key
	 *            the key
	 * @param locale
	 *            the locale
	 * @param parametros
	 *            the parametros
	 * @return the string
	 */
	public static String getString(Class<?> clazz, String key, Locale locale, String... parametros) {
		while (clazz != null) {
			String bundleName = clazz.getName();
			String strValue = getString(bundleName, key, locale, parametros);
			// Se ha encontrado si el valor no coincide con la clave...
			if (!StringUtils.equals(key, strValue)) {
				return strValue;
			}
			// Actualizamos la clase
			clazz = clazz.getSuperclass();
		}
		return key;
	}
	
}
