/**
 * Universitat Oberta de Catalunya
 * Ingeniera de Informtica
 * Proyecto fin de carrera
 * 
 * @author      Miguel Arias Fernndez 
 * @version     1.0    
 * 
 */
package es.uoc.maf.core.servlet;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.uoc.maf.core.action.utils.ActionContext;
import es.uoc.maf.core.action.utils.ActionProcessor;
import es.uoc.maf.core.config.Configuration;
import es.uoc.maf.core.config.ConfigurationFactory;

/**
 * The Class CoreServlet.
 * 
 * @author Miguel Arias Fernndez
 * @version     1.0 
 */
public class CoreServlet extends HttpServlet {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -8543471497725999911L;
	
	/** The Constant LOG. */
	private static final Logger LOG = LoggerFactory.getLogger(CoreServlet.class);
	
	// Mapa para almacenar la configuracin en funcin del nombre del Servlet
	// (por si hay varios servlets de este tipo)
	/** The configurations map. */
	private static Map<String, Configuration> configurationsMap = Collections.synchronizedMap(new HashMap<String, Configuration>());
	
	// Variable con el contexto del servlet 
	/** The servlet context. */
	private static ThreadLocal<ServletContext> SERVLET_CONTEXT = new InheritableThreadLocal<ServletContext>();
	
	/**
	 * Gets the context.
	 * 
	 * @return the context
	 */
	public static ServletContext getContext() {
		return SERVLET_CONTEXT.get();
	}
	
	/**
	 * Adds the configuration.
	 * 
	 * @param config
	 *            the config
	 */
	private void addConfiguration(Configuration config) {
		synchronized (configurationsMap) {
			configurationsMap.put(getServletName(), config);
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		LOG.debug("Iniciando servlet {} para path {}",getServletName(),getServletContext().getContextPath());
		try {
			// Primero establecemos el contexto del servlet para la factoria
			SERVLET_CONTEXT.set(getServletContext());
			// Cargamos la configuracion
			Configuration config = ConfigurationFactory.getInstance().initConfig();
			LOG.debug("Configuracin: {}",config);
			addConfiguration(config);
		}
		finally {
			// Limpiamos el contexto
			SERVLET_CONTEXT.remove();
		}
	}

	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		processRequest(req,resp);
	}

	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)	throws ServletException, IOException {
		processRequest(req,resp);
	}

	/**
	 * Process request.
	 * 
	 * @param req
	 *            the req
	 * @param resp
	 *            the resp
	 */
	private void processRequest(HttpServletRequest req, HttpServletResponse resp) {
		try {
			LOG.debug("Iniciando procesado de peticin.");
			// Creamos el contexto de la accin
			ActionContext actionContext = new ActionContext(req, resp, getServletContext());
			// Delegamos la ejecucion de la accin
			ActionProcessor.processAction(actionContext, configurationsMap.get(getServletName()));
		}
		finally {
			LOG.debug("Fin procesado de peticin.");
		}
	}

}
